﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Media;

namespace SudokuMachine
{
    class BlockBase
    {
        protected MatrixView _matrix;

        protected int _row;
        protected int _col;

        public MatrixView Matrix { get { return _matrix; } }

        public int Row { get { return _row; } }
        public int Col { get { return _col; } }

        public int Index { get { return _row * 3 + _col; } }

        public BlockBase(MatrixView matrix, int row, int col)
        {
            _matrix = matrix;

            _row = row;
            _col = col;
        }
    }

    class BlockView : BlockBase
    {
        private CellView[,] _cells;
        public CellView this[int row, int col] { get { return _cells[row, col]; } }

        private Border _border;
        public Decorator Chrome { get { return _border; } }

        private BlockHub _hub;
        public BlockHub Hub { get { return _hub; } }

        public BlockView(MatrixView matrix, int row, int col, bool isActive)
            : base(matrix, row, col)
        {
            _cells = new CellView[3, 3];

            for (int i = 0; i < 3; i++)
                for (int j = 0; j < 3; j++)
                {
                    _cells[i, j] = new CellView(this, i, j, isActive);
                }

            var layout = new UniformGrid { Rows = 3, Columns = 3 };
            foreach (var cell in _cells)
            {
                layout.Children.Add(cell.Chrome);
            }

            _border = new Border
            {
                Child = layout,

                Padding = new Thickness(1),
                Margin = new Thickness(1),
            };

            _hub = (isActive) ? new BlockHub(this) : null;
        }

        internal CellView[] GetCells()
        {
            var cells = new CellView[9];

            int index = 0;
            for (int i = 0; i < 3; i++)
                for (int j = 0; j < 3; j++)
                    {
                        cells[index++] = _cells[i, j];
                    }

            return cells;
        }

        internal void Hightlight(bool isOn)
        {
            foreach (var cell in _cells)
            {
                cell.Hightlight(isOn);
            }

            _border.BorderBrush = (isOn) ? Brushes.Black : null;
            _border.BorderThickness = (isOn) ? new Thickness(2) : new Thickness();

            _border.Padding = (isOn) ? new Thickness() : new Thickness(1);
            _border.Margin = (isOn) ? new Thickness() : new Thickness(1);
        }
    }

    class BlockHub
    {
        private CellView _cell;
        private KegBank _bank;

        public int CellRow { get { return _cell.Row; } }
        public int CellCol { get { return _cell.Col; } }

        public BlockHub(BlockView block)
        {
            _cell = block[0, 0];
        }

        internal void Bind(KegBank bank)
        {
            _bank = bank;
        }

        internal void Shift()
        {
            _cell.Content.Shift();
            _bank.Shift();
        }

        internal void Pass(CellView target)
        {
            _cell.Pass(target);
            _bank.Pass(target.Index);

            _cell = target;
        }

        internal void Reset(int value)
        {
            _cell.Setup(0);
            _cell.Content.Hightlight(false);

            int row = _cell.Block.Col;
            int col = _cell.Block.Row;

            _cell = _cell.Block[row, col];
            _cell.Setup(value);

            int index = row * 3 + col;
            _bank.Reset(index, value);
        }

        internal void CheckRow(CellView target, int destRow)
        {
            if (target.Row == _cell.Row)
                Pass(_cell.Block[destRow, _cell.Col]);
        }

        internal void CheckCol(CellView target, int destCol)
        {
            if (target.Col == _cell.Col)
                Pass(_cell.Block[_cell.Row, destCol]);
        }

        internal void HightlightContent(bool isOn)
        {
            _cell.Content.Hightlight(isOn);
        }
    }
}