﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;

namespace SudokuMachine
{
    class CellBase
    {
        protected BlockView _block;

        protected int _row;
        protected int _col;

        public BlockView Block { get { return _block; } }
        public MatrixView Matrix { get { return _block.Matrix; } }

        public int Row { get { return _row; } }
        public int Col { get { return _col; } }

        public int Index { get { return _row * 3 + _col; } }

        public CellBase(BlockView block, int row, int col)
        {
            _block = block;

            _row = row;
            _col = col;
        }
    }

    class CellView : CellBase
    {
        private CellContent _content;
        public CellContent Content { get { return _content; } }

        private Border _border;
        public Decorator Chrome { get { return _border; } }

        public CellView(BlockView block, int row, int col, bool isActive)
            : base(block, row, col)
        {
            _content = new CellContent(isActive);

            _border = new Border
            {
                Child = _content.Display,

                Background = Brushes.White,
                BorderBrush = (isActive) ? Brushes.DimGray : Brushes.Black,
                BorderThickness = new Thickness(1),

                Margin = new Thickness(1),
            };

            if (isActive)
            {
                _border.MouseLeftButtonDown += new MouseButtonEventHandler(OnMouseLeftDown);

                _border.MouseEnter += new MouseEventHandler(OnMouseEnter);
                _border.MouseLeave += new MouseEventHandler(OnMouseLeave);
            }
        }

        internal void Hit()
        {
            if (_content.Value == 0) Matrix.Drift(this);
            else
                Matrix.Shift();
        }

        private void OnMouseLeftDown(object sender, MouseButtonEventArgs e)
        {
            Hit();
        }

        private void OnMouseEnter(object sender, MouseEventArgs e)
        {
            if (_content.Value != 0) Matrix.HightlightContent(true);
        }

        private void OnMouseLeave(object sender, MouseEventArgs e)
        {
            if (_content.Value != 0) Matrix.HightlightContent(false);
        }

        internal void Pass(CellView target)
        {
            target.Setup(_content.Value);
            target.Content.Hightlight(true);
            Setup(0);
        }

        internal void Setup(int value)
        {
            _content.Setup(value);
            _border.Background = (value != 0) ? Brushes.Khaki : Brushes.White;
        }

        internal void Flash(bool isOn)
        {
            _content.Flash(isOn);
            _border.Background = (isOn) ? Brushes.Gold : Brushes.White;
            _border.CornerRadius = (isOn) ? new CornerRadius(MatrixBase.Corner) : new CornerRadius();
        }

        internal void Hightlight(bool isOn)
        {
            _border.BorderBrush = (isOn) ? Brushes.Black : Brushes.DimGray;
            _border.Margin = (isOn) ? new Thickness() : new Thickness(1);
        }
    }

    class CellContent
    {
        private int _value;
        public int Value { get { return _value; } }

        private TextBlock _display;
        public TextBlock Display { get { return _display; } }

        private static readonly FontFamily FONT_FAMILY = new FontFamily("Comic Sans MS");
        private static readonly string[] VALUE_TEXT = {"", "1", "2", "3", "4", "5", "6", "7", "8", "9", "$"};

        public static FontFamily FontFamily { get { return FONT_FAMILY; } }

        public CellContent(bool isActive)
        {
            _value = 0;

            _display = new TextBlock
            {
                FontFamily = FONT_FAMILY,
                Foreground = (isActive) ? Brushes.Black : Brushes.Gray,
                FontSize = (isActive) ? 24 : 20,

                HorizontalAlignment = HorizontalAlignment.Center,
                VerticalAlignment = VerticalAlignment.Center,

                Text = VALUE_TEXT[_value]
            };
        }

        internal void Shift()
        {
            if (++_value > 9) _value = 1;
            _display.Text = VALUE_TEXT[_value];
            Hightlight(true);
        }

        internal void Setup(int value)
        {
            _value = value;
            _display.Text = VALUE_TEXT[_value];
        }

        internal void Flash(bool isOn)
        {
            Setup((isOn) ? 10 : 0);
            _display.Foreground = (isOn) ? Brushes.SaddleBrown : Brushes.Gray;
            _display.FontWeight = (isOn) ? FontWeights.Bold : FontWeights.Normal;
        }

        internal void Hightlight(bool isOn)
        {
            _display.FontWeight = (isOn) ? FontWeights.Bold : FontWeights.Normal;
        }
    }
}