﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Input;
using System.Windows.Media;

namespace SudokuMachine
{
    class MatrixBase
    {
        const int CELL_SIZE = 30;
        const int BLOCK_SIZE = CELL_SIZE * 3 + 4;
        const int MATRIX_SIZE = BLOCK_SIZE * 3 + 8;

        public static double Size { get { return MATRIX_SIZE; } }
        public static double Margin { get { return 12; } }
        public static double Corner { get { return 6; } }
    }

    class MatrixView : MatrixBase
    {
        private BlockView[,] _blocks;
        public BlockView this[int row, int col] { get { return _blocks[row, col]; } }

        private Border _border;
        public Decorator Chrome { get { return _border; } }

        private MatrixRose[] _roses;
        private BarSlot _countSlot;

        public MatrixView(BarSlot countSlot)
        {
            bool isActive = (countSlot != null);

            _blocks = new BlockView[3, 3];

            for (int i = 0; i < 3; i++)
                for (int j = 0; j < 3; j++)
                {
                    _blocks[i, j] = new BlockView(this, i, j, isActive);
                }

            var layout = new UniformGrid { Rows = 3, Columns = 3 };
            foreach (var block in _blocks)
            {
                layout.Children.Add(block.Chrome);
            }

            _border = new Border
            {
                Child = layout,

                Background = Brushes.WhiteSmoke,
                BorderBrush =  (isActive) ? Brushes.DimGray : Brushes.Black,
                BorderThickness = new Thickness(1),
                CornerRadius = new CornerRadius(MatrixBase.Corner),

                Padding = new Thickness(3),
                Margin = new Thickness(MatrixBase.Margin),

                Width = MatrixBase.Size,
                Height = MatrixBase.Size,
            };

            if (isActive)
            {
                _roses = new MatrixRose[9];

                int i = 0;
                foreach (var block in _blocks)
                {
                    _roses[i++] = new MatrixRose(block);
                }

                _countSlot = countSlot;
            }
            else
            {
                _roses = null;
                _countSlot = null;
            }
        }

        internal void Bind(KegBank[] banks)
        {
            int i = 0;
            foreach (var block in _blocks)
            {
                block.Hub.Bind(banks[i++]);
            }
        }

        internal CellView[] GetCells()
        {
            var cells = new CellView[81];

            int index = 0;
            for (int i = 0; i < 3; i++)
                for (int j = 0; j < 3; j++)
                    for (int m = 0; m < 3; m++)
                        for (int n = 0; n < 3; n++)
                        {
                            cells[index++] = _blocks[i, j][m, n];
                        }

            return cells;
        }

        internal BlockView[] GetBlocks()
        {
            var blocks = new BlockView[9];

            int index = 0;
            for (int i = 0; i < 3; i++)
                for (int j = 0; j < 3; j++)
                {
                    blocks[index++] = _blocks[i, j];
                }

            return blocks;
        }

        internal void Drift(CellView target)
        {
            MatrixRose rose;
            rose = _roses[target.Block.Index];

            rose.Drift(target);
            _countSlot.Drift();
        }

        internal void Shift()
        {
            foreach (var block in _blocks)
            {
                block.Hub.Shift();
            }
            _countSlot.Shift();
        }

        internal void Reset(int value)
        {
            foreach (var block in _blocks)
            {
                block.Hub.Reset(value);
            }
        }

        internal void Hightlight(bool isOn)
        {
            _border.BorderBrush = (isOn) ? Brushes.Black : Brushes.DimGray;
            _border.BorderThickness = (isOn) ? new Thickness(4) : new Thickness(1);
            _border.Padding = (isOn) ? new Thickness() : new Thickness(3);
        }

        internal void HightlightContent(bool isOn)
        {
            foreach (var block in _blocks)
            {
                block.Hub.HightlightContent(isOn);
            }
        }
    }

    class MatrixRose
    {
        private BlockView[] _blocks;
        public BlockView Central { get { return _blocks[0]; } }

        public MatrixRose(BlockView central)
        {
            _blocks = new BlockView[5];

            _blocks[0] = central;

            int col = central.Col - 1;
            if (col < 0) col = 2;
            _blocks[1] = central.Matrix[central.Row, col];

            col = central.Col + 1;
            if (col > 2) col = 0;
            _blocks[2] = central.Matrix[central.Row, col];

            int row = central.Row - 1;
            if (row < 0) row = 2;
            _blocks[3] = central.Matrix[row, central.Col];

            row = central.Row + 1;
            if (row > 2) row = 0;
            _blocks[4] = central.Matrix[row, central.Col];

            central.Chrome.MouseEnter += new MouseEventHandler(OnMouseEnter);
            central.Chrome.MouseLeave += new MouseEventHandler(OnMouseLeave);
        }

        internal void Drift(CellView target)
        {
            Central.Matrix.HightlightContent(false);

            int row = Central.Hub.CellRow;

            if (target.Row != row)
            {
                _blocks[1].Hub.CheckRow(target, row);
                _blocks[2].Hub.CheckRow(target, row);
            }

            int col = Central.Hub.CellCol;

            if (target.Col != col)
            {
                _blocks[3].Hub.CheckCol(target, col);
                _blocks[4].Hub.CheckCol(target, col);
            }

            Central.Hub.Pass(target);
        }

        private void OnMouseEnter(object sender, MouseEventArgs e)
        {
            Hightlight(true);
        }

        private void OnMouseLeave(object sender, MouseEventArgs e)
        {
            Hightlight(false);
        }

        internal void Hightlight(bool isOn)
        {
            Central.Matrix.Hightlight(isOn);

            foreach (var block in _blocks)
            {
                block.Hightlight(isOn);
            }
        }
    }
}