﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;

namespace SudokuMachine
{
    class StatusBar
    {
        private BlenderSlot _amountSlot;
        private BowlSlot _countSlot1;
        private BowlSlot _countSlot2;

        public BlenderSlot AmountSlot { get { return _amountSlot; } }
        public BowlSlot CountSlot1 { get { return _countSlot1; } }
        public BowlSlot CountSlot2 { get { return _countSlot2; } }

        private StackPanel _layout;
        public Panel Layout { get { return _layout; } }

        public StatusBar()
        {
            _amountSlot = new BlenderSlot();
            _countSlot1 = new BowlSlot(_amountSlot);
            _countSlot2 = new BowlSlot(_amountSlot);

            _layout = new StackPanel { Orientation = Orientation.Horizontal };
            _layout.Children.Add(_countSlot1.Chrome);
            _layout.Children.Add(_amountSlot.Chrome);
            _layout.Children.Add(_countSlot2.Chrome);
        }

        internal void Reset()
        {
            _amountSlot.Reset();
            _countSlot1.Reset();
            _countSlot2.Reset();
        }
    }

    abstract class BarSlot
    {
        private TextBlock _display;

        private Border _border;
        public Decorator Chrome { get { return _border; } }

        public BarSlot(bool isCentral)
        {
            _display = new TextBlock
            {
                FontFamily = CellContent.FontFamily,
                FontSize = 32,
                Foreground = (isCentral) ? Brushes.Maroon : Brushes.Black,
                FontWeight = (isCentral) ? FontWeights.Bold : FontWeights.Normal,

                HorizontalAlignment = HorizontalAlignment.Center,
                VerticalAlignment = VerticalAlignment.Center,
            };

            var inner = new Border
            {
                Child = _display,

                Background = (isCentral) ? Brushes.Gold : Brushes.Khaki,
                BorderBrush = (isCentral) ? Brushes.Black : Brushes.DimGray,
                BorderThickness = new Thickness(1),
                CornerRadius = new CornerRadius(MatrixBase.Corner),

                Margin = new Thickness(6),
            };

            _border = new Border
            {
                Child = inner,

                Background = Brushes.White,
                BorderBrush = (isCentral) ? Brushes.Black : Brushes.DimGray,
                BorderThickness = new Thickness(1),
                CornerRadius = new CornerRadius(MatrixBase.Corner),

                Margin = new Thickness(MatrixBase.Margin),

                Width = MatrixBase.Size,
                Height = 65,
            };
        }

        protected void Update(string text)
        {
            _display.Text = text;
        }

        internal abstract void Drift();
        internal abstract void Shift();
    }

    class BlenderSlot : BarSlot
    {
        private int _count;
        private int _gain;
        private int _amount;

        public BlenderSlot()
            : base(true) { }

        private void Update()
        {
            Update(string.Format("$ {0}", _amount));
        }

        internal void Inc()
        {
            _count++;
        }

        internal void Dec()
        {
            _count--;
        }

        internal override void Drift()
        {
            _gain = (_count > 0) ? _count : -1;
            _amount += _gain;

            Update();
        }

        internal override void Shift()
        {
            _gain = _count;
            _amount += _gain;

            Update();
        }

        internal void Reset()
        {
            _count = 9;
            _gain = 0;
            _amount = 0;

            Update();
        }
    }

    class BowlSlot : BarSlot
    {
        private int _driftCount;
        private int _shiftCount;

        private BarSlot _amountSlot;

        public BowlSlot(BarSlot amountSlot)
            : base(false)
        {
            _amountSlot = amountSlot;
        }

        private void Update()
        {
            Update(string.Format("{0}  :  {1}", _driftCount, _shiftCount));
        }

        internal override void Drift()
        {
            _driftCount++;
            _amountSlot.Drift();

            Update();
        }

        internal override void Shift()
        {
            _shiftCount++;
            _amountSlot.Shift();

            Update();
        }

        internal void Reset()
        {
            _driftCount = 0;
            _shiftCount = 0;

            Update();
        }
    }
}